package {
    
    import flash.display.DisplayObject;
    import flash.events.Event;
    import flash.events.EventPhase;
    import flash.events.MouseEvent;
    
    import mx.controls.CheckBox;
    import mx.controls.Label;
    import mx.controls.List;
    import mx.controls.listClasses.BaseListData;
    import mx.controls.listClasses.IDropInListItemRenderer;
    import mx.controls.listClasses.IListItemRenderer;
    import mx.controls.listClasses.ListData;
    import mx.core.IFlexDisplayObject;
    import mx.core.UIComponent;
    import mx.events.FlexEvent;

    [Event(name="dataChange", type="mx.events.FlexEvent")]
    public class CheckBoxListItemRenderer extends UIComponent implements IListItemRenderer, IDropInListItemRenderer
    {
        public var checkBox:CheckBox;

        public var imageIcon:ImageIcon;

        public var label:Label;

        protected var icon:IFlexDisplayObject;

        private var listOwner:List;

        private var _data:Object;

        [Bindable("dataChange")]
        public function get data():Object
        {
            return _data;
        }

        public function set data(value:Object):void
        {
            _data = value;
            invalidateProperties();
            dispatchEvent(new FlexEvent(FlexEvent.DATA_CHANGE));
        }

        private var _listData:ListData;

        public function get listData():BaseListData{
            return _listData;
        }

        public function set listData(value:BaseListData):void{
            _listData = ListData(value);
            if(_listData)
            {
                listOwner = List(_listData.owner);
            }
            invalidateProperties();
        }

        public override function get baselinePosition():Number{
            return label.baselinePosition;
        }

        public function CheckBoxListItemRenderer()
        {
            super();
        }

        private function mouseDownHandler( event:Event ):void{
            //trace(event);
            event.stopImmediatePropagation();
            event.preventDefault();
        }

        private function mouseUpHandler( event:Event ):void{
            //trace(event);
            event.stopImmediatePropagation();
            event.preventDefault();
        }

        private function clickHandler( event:Event ):void{
            if( event.eventPhase != EventPhase.AT_TARGET ){
                //trace(event);
                event.stopImmediatePropagation();
                event.preventDefault();
            }
        }

        private function labelClickHandler(event:MouseEvent):void{
            checkBox.selected = !checkBox.selected;
            data["selected"] = checkBox.selected;
            event.stopImmediatePropagation();
            event.preventDefault();
            dispatchEvent(event);
        }

        private function checkBoxChangeHandler(event:Event):void{
            data["selected"] = checkBox.selected;
            event.stopImmediatePropagation();
        }

        protected override function createChildren():void{
            super.createChildren();

            if (!checkBox)
            {
                checkBox = new CheckBox();
                checkBox.addEventListener(Event.CHANGE,checkBoxChangeHandler);
                checkBox.styleName = this;
                addChild(checkBox);
            }

            if (!imageIcon) {
                imageIcon = new ImageIcon();
                addChild(imageIcon);
            }

            if (!label)
            {
                label = new Label();
                label.styleName = this;
                label.addEventListener(MouseEvent.CLICK,labelClickHandler);
                addChild(label);
            }
            addEventListener(MouseEvent.MOUSE_DOWN,mouseDownHandler,false,int.MAX_VALUE);
            addEventListener(MouseEvent.MOUSE_UP,mouseUpHandler,false,int.MAX_VALUE);
            addEventListener(MouseEvent.CLICK,clickHandler,false,int.MAX_VALUE);
        }

        protected override function commitProperties():void{
            super.commitProperties();

            if(icon)
            {
                removeChild(DisplayObject(icon));
                icon = null;
            }

            if(_data != null)
            {
                if(_listData.icon)
                {
                    var iconClass:Class = _listData.icon;
                    icon = new iconClass();

                    addChild(DisplayObject(icon));
                }

                checkBox.selected = data["selected"];

                label.text = _listData.label ? _listData.label : " ";

                if(listOwner.showDataTips)
                {
                    toolTip = checkBox.toolTip;
                }
                else
                {
                    toolTip = null;
                }

                imageIcon.styleName = data["icon"];

            }
            else
            {
                checkBox.label = " ";
                toolTip = null;
            }
        }

        protected override function measure():void{
            super.measure();

            var w:Number = 0;

            if(icon)
            {
                w = icon.measuredWidth;
            }

            if(checkBox.width < 4 || checkBox.height < 4)
            {
                checkBox.width = 16;
                checkBox.height = 16;

            }

            if(isNaN(explicitWidth))
            {
                w += checkBox.getExplicitOrMeasuredWidth();
                w += imageIcon.getExplicitOrMeasuredWidth();
                w += label.getExplicitOrMeasuredWidth();
                measuredWidth = w;
                measuredHeight = checkBox.getExplicitOrMeasuredHeight();
                measuredHeight = Math.max(measuredHeight,label.getExplicitOrMeasuredHeight());
            }
            else
            {
                measuredWidth = explicitWidth;
                label.setActualSize(explicitWidth - w - 16, label.height);
                measuredHeight = checkBox.getExplicitOrMeasuredHeight();
                measuredHeight = Math.max(measuredHeight,label.getExplicitOrMeasuredHeight());
                if(icon && icon.measuredHeight > measuredHeight)
                {
                    measuredHeight = icon.measuredHeight;
                }
            }
        }

        protected override function updateDisplayList(unscaledWidth:Number, unscaledHeight:Number):void{
            super.updateDisplayList(unscaledWidth, unscaledHeight);

            var xPosition:Number = 0;
            var xGap:int = 10;

            checkBox.setActualSize(16, 16);
            checkBox.move(xPosition + 8, (unscaledHeight - checkBox.getExplicitOrMeasuredHeight()) / 2 );

            if(imageIcon) {
                xPosition = checkBox.x + checkBox.measuredWidth + xGap + 3;
                imageIcon.setActualSize(imageIcon.measuredWidth, imageIcon.measuredHeight);
                imageIcon.move(xPosition, (unscaledHeight - imageIcon.getExplicitOrMeasuredHeight()) / 2 );
            }

            xPosition = checkBox.x + checkBox.measuredWidth + imageIcon.measuredWidth + xGap + 3;

            label.setActualSize(unscaledWidth - xPosition, measuredHeight);
            label.move(xPosition + 8, (unscaledHeight - label.getExplicitOrMeasuredHeight()) / 2 + 2);


        }
    }
}